@@ -252,11 +252,24 @@ class KitsuConfig(flow.Object):
             child_value.set(data["id"])
 
 
+class DefaultFiles(flow.Object):
+
+    file_names = flow.Param('')
+
+
+class DefaultFilesMap(flow.Map):
+
+    @classmethod
+    def mapped_type(cls):
+        return DefaultFiles
+
+
 class ProjectSettings(flow.Object):
     project_nice_name = flow.Param()
     project_thumbnail = flow.Param().ui(editor='textarea')
     support = flow.Param("")
 
+    default_files = flow.Child(DefaultFilesMap)
     non_editable_files = flow.Param("")
 
 
@@ -350,8 +363,6 @@ class Project(flow.Object):
     admin = flow.Child(Admin)
     synchronization = flow.Child(Synchronization).ui(expanded=True)
 
-    _show_login_page = flow.Computed().ui(editor="bool")
-
     _RUNNERS_FACTORY = None
 
     def get_root(self, alternative=None):
@@ -427,31 +438,28 @@ class Project(flow.Object):
             # Unregistered user not admin by default
             return False
 
-    def compute_child_value(self, child_value):
-        if child_value is self._show_login_page:
-            kitsu_api = self.kitsu_api()
-            valid_host = kitsu_api.host_is_valid()
+    def show_login_page(self):
+        kitsu_api = self.kitsu_api()
+        valid_host = kitsu_api.host_is_valid()
 
-            if not valid_host:
+        if not valid_host:
+            logged_in = False
+        else:
+            logged_in = kitsu_api.current_user_logged_in()
+
+        if not logged_in:
+            try:
+                f = open("%s/kitsu_config.json" % self.user_settings_folder(), "r")
+            except IOError:
                 logged_in = False
             else:
+                kitsu_config = json.load(f)
+                kitsu_api.set_host(kitsu_config["kitsu_host"])
+                kitsu_api.set_tokens(kitsu_config["tokens"])
+
                 logged_in = kitsu_api.current_user_logged_in()
 
-            if not logged_in:
-                try:
-                    f = open("%s/kitsu_config.json" % self.user_settings_folder(), "r")
-                except IOError:
-                    logged_in = False
-                else:
-                    kitsu_config = json.load(f)
-                    kitsu_api.set_host(kitsu_config["kitsu_host"])
-                    kitsu_api.set_tokens(kitsu_config["tokens"])
-
-                    logged_in = kitsu_api.current_user_logged_in()
-
-            child_value.set(not kitsu_api.host_is_valid() or not logged_in)
-        else:
-            super(Project, self).compute_child_value(child_value)
+        return not kitsu_api.host_is_valid() or not logged_in
 
     def update_kitsu_host(self, server_url):
         kitsu_api = self.kitsu_api()
@@ -611,5 +619,5 @@ class Project(flow.Object):
 
         self.touch()
 
-        if self._show_login_page.get():
+        if self.show_login_page():
             ui["custom_page"] = "libreflow.baseflow.LoginPageWidget"
