@@ -172,7 +172,6 @@ class PublishFileAction(LaunchSessionWorker):
     ICON = ("icons.libreflow", "publish")
 
     _file = flow.Parent()
-    _department = flow.Parent(3)
 
     comment = flow.SessionParam("")
     upload_after_publish = flow.BoolParam(False)
@@ -252,8 +251,7 @@ class PublishFileAction(LaunchSessionWorker):
             self._file.set_current_user_on_revision(published_revision.name())
             self._file.unlock()
 
-        if self._department.auto_current.get():
-            published_revision.make_current.run(None)
+        published_revision.make_current.run(None)
 
         self._file.touch()
 
@@ -1507,7 +1505,7 @@ class TrackedFile(File):
         )
 
         try:
-            os.mkdir(head_revision.path.get())
+            os.makedirs(head_revision.path.get())
         except OSError:
             self.root().session().log_error(
                 "Creation of head revision folder '{}' failed\n\n==> TRACEBACK\n".format(
