@@ -2,11 +2,13 @@ import os
 import sys
 import gazu
 import time
-import datetime
+from datetime import datetime, timedelta
 import subprocess
 import platform
 from minio import Minio
 import zipfile
+import fnmatch
+import re
 
 from kabaret import flow
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
@@ -68,18 +70,6 @@ class Job(flow.Object):
         return job_repr
 
 
-class ClearQueueAction(flow.Action):
-
-    _queue = flow.Parent()
-
-    def needs_dialog(self):
-        return False
-    
-    def run(self, button):
-        self._queue.clear()
-        self._queue.touch()
-
-
 class JobQueue(flow.Map):
 
     @classmethod
@@ -161,7 +151,7 @@ class JobQueue(flow.Map):
         row["Type"] = item.type.get()
         row["Status"] = item.status.get()
         row["Priority"] = item.priority.get()
-        row["Emitted on"] = datetime.datetime.fromtimestamp(item.date.get()).ctime()
+        row["Emitted on"] = datetime.fromtimestamp(item.date.get()).ctime()
         row["Expires on"] = "Never" if item.date_end.get() == -1 else item.date_end.get()
         row["Requested by (site/user)"] = "%s/%s" % (
             item.requested_by_studio.get(),
@@ -684,6 +674,46 @@ class GotoQueue(flow.Action):
         return self.get_result(goto=self._site.queue.oid())
 
 
+class ClearQueueJobStatus(flow.values.ChoiceValue):
+
+    CHOICES = JobStatus.CHOICES + ["All"]
+
+
+class ClearQueueLoadType(flow.values.ChoiceValue):
+
+    CHOICES = LoadType.CHOICES + ["All"]
+
+
+class ClearQueueAction(flow.Action):
+
+    _site = flow.Parent()
+    status = flow.Param("PROCESSED", ClearQueueJobStatus)
+    type = flow.Param("All", ClearQueueLoadType)
+    emitted_before = flow.SessionParam((datetime.now() - timedelta(days=1)).timestamp()).ui(
+        editor='datetime'
+    )
+
+    def get_buttons(self):
+        self.message.set(
+            "<h2>Clear %s jobs</h2>" % self._site.name()
+        )
+        return ["Clear", "Cancel"]
+    
+    def run(self, button):
+        queue = self._site.get_queue()
+        status = self.status.get()
+        type = self.type.get()
+
+        if status == "All" and type == "All":
+            queue.clear()
+        else:
+            for job in queue.jobs(status=self.status.get()):
+                if job.date.get() < self.emitted_before.get():
+                    queue.remove(job.name())
+
+        queue.touch()
+
+
 class WorkingSite(Site):
     
     site_type = flow.Param("Studio", StaticSiteTypeChoices)
@@ -708,6 +738,7 @@ class WorkingSite(Site):
     goto_queue = flow.Child(GotoQueue).ui(
         label="Show job queue"
     )
+    clear_queue = flow.Child(ClearQueueAction)
 
     def compute_child_value(self, child_value):
         if child_value is self.root_folder:
@@ -871,3 +902,175 @@ class SyncMap(flow.DynamicMap):
 
         if item.name() == self.version.site.get():
             row['Name'] += " (S)"
+
+
+class WorkingSiteChoiceValue(flow.values.ChoiceValue):
+
+    _parent = flow.Parent()
+
+    def choices(self):
+        return self.root().project().get_working_sites().mapped_names()
+
+
+class RequestRevisionsAs(flow.Action):
+
+    pattern = flow.SessionParam("").watched().ui(
+        placeholder="Revision oid pattern"
+    )
+    requesting_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
+        label="Requesting site"
+    )
+    requested_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
+        label="Site to query"
+    )
+    revision_oids = flow.SessionParam("").ui(
+        editor="textarea",
+        html=True,
+        editable=False,
+    )
+
+    def allow_context(self, context):
+        current_site = self.root().project().get_current_site()
+        return context and current_site.request_files_from_anywhere.get()
+    
+    def _title(self):
+        return "Request revisions"
+    
+    def child_value_changed(self, child_value):
+        if child_value in (self.requesting_site_name, self.requested_site_name):
+            msg = "<h2>%s</h2>" % self._title()
+            if self.requested_site_name.get() == self.requesting_site_name.get():
+                msg += (
+                    "<font color=#D5000D>"
+                    "Requested and requesting sites can't be the same."
+                    "</font>"
+                )
+
+            self.message.set(msg)
+        elif child_value is self.pattern:
+            self._oids = self._get_oid(self.pattern.get())
+            self.revision_oids.set("<br>".join(self._oids))
+    
+    def _get_oid(self, pattern_str):
+        oids = []
+        patterns = []
+        
+        for pattern in pattern_str.split(';'):
+            patterns += self.resolve_pattern(pattern)
+        
+        for pattern in patterns:
+            oids += self.glob(self.root().project().oid(), pattern, 0)
+        
+        return oids
+
+    def get_buttons(self):
+        self.requested_site_name.set(self.requested_site_name.choices()[0])
+        self.revision_oids.set("")
+        self._oids = []
+
+        return ["Request", "Close"]
+    
+    def ls(self, root_oid):
+        related_info, mapped_names = self.root().session().cmds.Flow.ls(root_oid)
+        relation_oids = [rel_info[0] for rel_info in related_info]
+        mapped_oids = ["%s/%s" % (root_oid, name) for name in mapped_names]
+        
+        return relation_oids + mapped_oids
+    
+    def get_last_publication(self, file_oid):
+        o = self.root().get_object(file_oid)
+        
+        try:
+            head = o.get_head_revision()
+        except AttributeError:
+            return None
+        
+        if not head:
+            return None
+        
+        return head.name()
+    
+    def glob(self, root_oid, pattern, level):
+        if level >= pattern.count("/") - 1:
+            return [root_oid]
+
+        matches = []
+        level_pattern = "/".join(pattern.split("/")[:level + 3])
+
+        if level_pattern.endswith("[last]"):
+            file_oid = self.root().session().cmds.Flow.resolve_path(root_oid + "/../..")
+            head_name = self.get_last_publication(file_oid)
+
+            if not head_name:
+                return []
+            
+            pattern = pattern.replace("[last]", head_name)
+            level_pattern = level_pattern.replace("[last]", head_name)
+        
+        for oid in self.ls(root_oid):
+            if fnmatch.fnmatch(oid, level_pattern):
+                matches += self.glob(oid, pattern, level + 1)
+        
+        return matches
+    
+    def resolve_pattern(self, pattern_oid):
+        '''
+        Returns a list of patterns which correspond to all substitution combinations of `pattern_oid`.
+        Substitute string are specified in `pattern_oid` between brackets and separated with comas (`{s0, s1, ...}`).
+        '''
+        match = re.search(r'{[^{}]+}', pattern_oid)
+
+        if not match:
+            return [pattern_oid]
+
+        substitutes = match.group()[1:-1].split(',')
+        substitutes = [sub.replace(' ', '') for sub in substitutes]
+
+        sub_oids = [pattern_oid.replace(match.group(), sub, 1) for sub in substitutes]
+        oids = []
+
+        for sub_oid in sub_oids:
+            oids += self.resolve_pattern(sub_oid)
+        
+        return oids
+
+    
+    def run(self, button):
+        if button == "Close":
+            return
+        
+        if not self._oids:
+            self._oids = self._get_oid(self.pattern.get())
+
+        objects = [self.root().get_object(oid) for oid in self._oids]
+        processed_oids = []
+        
+        for o in objects:
+            oid = o.oid()
+
+            if hasattr(o, 'request_as') and o.ready_for_sync.get():
+                requested_site_name = self.requested_site_name.get()
+                requesting_site_name = self.requesting_site_name.get()
+
+                if not requesting_site_name in o.request_as.requesting_site_name.choices():
+                    oid += f" <font color='red'>Available on {requesting_site_name}</font>"
+                elif not requested_site_name in o.request_as.requested_site_name.choices():
+                    oid += f" <font color='red'>Not available on {requested_site_name}</font>"
+                else:
+                    o.request_as.requested_site_name.set(requested_site_name)
+                    o.request_as.requesting_site_name.set(requesting_site_name)
+                    o.request_as.run(None)
+                    oid += " <font color='green'>OK</font>"
+            else:
+                if not o.ready_for_sync.get():
+                    msg = "Revision not ready for sync"
+                else:
+                    msg = "Not a file revision"
+                
+                oid += " <font color='red'>%s</font>" % msg
+            
+            processed_oids.append(oid)
+        
+        self.revision_oids.set("<br>".join(processed_oids))
+
+        return self.get_result(close=False)
